import sys
from importlib import metadata


def lab_version():
    return metadata.version('lab-partner')


def is_linux() -> bool:
    """
    Check current platform is Linux
    :return: True on Linux
    """
    return sys.platform in ('linux',)


def is_supported_platform() -> bool:
    """
    Check current platform is MacOS or Linux
    :return: True on MacOS or Linux
    """
    return sys.platform in ('darwin', 'linux')



import logging
import json
import shlex
from subprocess import (
    Popen,
    PIPE,
    CalledProcessError,
    check_call,
    DEVNULL
)
from typing import Iterable, Dict, Any, List, Optional

logger = logging.getLogger(__name__)


def run_process_quiet(command: str, env: Optional[Dict[str, str]] = None):
    check_call(shlex.split(command), stdout=DEVNULL, stderr=DEVNULL, env=env)


def run_process_stream_result(command: str, env: Optional[Dict[str, str]] = None) -> Iterable[str]:
    """
    Run subprocess that streams stdout and stderr to handler functions
    :param command: the command line string to be executed
    :return: a generator of each returned line from STDOUT
    """
    err = None
    with Popen(shlex.split(command), stdout=PIPE, stderr=PIPE, bufsize=1, universal_newlines=True, env=env, encoding='utf-8') as p:
        if p.stdout:
            for line in p.stdout:
                if line:
                    yield line
        if p.stderr:
            err = p.stderr.read()
    if p.returncode != 0 and err:
        for line in err.split('\n'):
            logger.error(line)
        raise CalledProcessError(p.returncode, p.args, stderr=err)


def process_output_as_json(process_output: Iterable[str]) -> List[Dict[str, Any]]:
    """
    Convert the results of a subprocess to a json generator
    :param process_output: output from running `run_process(...)`
    :return:
    """
    return [json.loads(output) for output in process_output]


def run_process(command: str, env: Optional[Dict[str, str]] = None) -> List[str]:
    return list(run_process_stream_result(command, env))


def run_process_single_result(command: str, env: Optional[Dict[str, str]] = None) -> str:
    return run_process(command, env)[0]




#!/usr/bin/env python

import subprocess
import sys
import os
from pathlib import Path
from typing import Dict
import click
from click import ParamType, Option
from click.types import IntParamType


opensearch_data = os.path.join(workspace_data, 'opensearch', 'data')
opensearch_logs = os.path.join(workspace_data, 'opensearch', 'logs')

Path(opensearch_data).mkdir(parents=True, exist_ok=True)
Path(opensearch_logs).mkdir(parents=True, exist_ok=True)


env = {
    'PATH': path,
    'USER': user,
    'UID': uid,
    'GID': gid,
    'HOME': home,
    'DISPLAY': display,
    'WORKSPACE': workspace,
    'WORKSPACE_DATA': workspace_data,
    'NETWORK_NAME': network_name,
    'OPENSEARCH_DATA': opensearch_data,
    'OPENSEARCH_LOGS': opensearch_logs,
    'LAB_VERSION': lab_version
}


@click.group()
def services():
    pass


def expand_variables(content: str, replace_vars: Dict[str, str]):
    for k, v in replace_vars.items():
        content = content.replace('$' + k, v)
        content = content.replace('${' + k + '}', v)
    return content


def parse_image_name(image_name: str):
    if '/' in image_name:
        image_name_parts = image_name.split('/')
        registry_name = image_name_parts[0]
        root_image_name = image_name_parts[1].split(':')[0]
    else:
        registry_name = None
        root_image_name = image_name.split(':')[0]
    image_version = image_name.split(':')[1]
    return registry_name, root_image_name, image_version


def read_docker_compose_config() -> str:
    try:
        with open('/opt/lab-partner/docker-compose.yml', 'r') as config:
            prepared_config = expand_variables(config.read(), env)
        return prepared_config
    except subprocess.CalledProcessError as e:
        click.echo(e.stderr)
        sys.exit('Error parsing docker-compose.yml')


def exec_compose(compose_cmd: str, config: str):
    r, w = os.pipe()
    pid = os.fork()
    if pid == 0:
        # Child process
        os.dup2(r, sys.stdin.fileno())
        os.close(r)
        os.close(w)
        up_command = f'docker-compose -f - {compose_cmd}'.split()
        os.execvp(up_command[0], up_command)
    else:
        # Parent process
        os.close(r)
        os.write(w, bytearray(config, 'utf-8'))
        os.close(w)
        pid, status = os.waitpid(pid, 0)


TRACING_SERVICES = 'proxy data-prepper otel-collector jaeger-agent opensearch-dashboards opensearch'
NOTEBOOK_SERVICES = 'proxy jupyter'


def start_services(services: str = ''):
    config = read_docker_compose_config()
    exec_compose(f'up -d --remove-orphans --force-recreate {services}', config)


def stop_services(services: str = ''):
    config = read_docker_compose_config()
    exec_compose(f'stop {services}', config)
    exec_compose(f'rm --force --stop -v {services}', config)


@services.command()
def start_all():
    start_services()


@services.command()
def stop_all():
    stop_services()


@services.command()
def start_tracing():
    start_services(TRACING_SERVICES)


@services.command()
def stop_tracing():
    stop_services(TRACING_SERVICES)


@services.command()
def start_notebook():
    start_services(NOTEBOOK_SERVICES)


@services.command()
def stop_notebook():
    stop_services(NOTEBOOK_SERVICES)


if __name__ == '__main__':
    services()



#!/usr/bin/env python

import logging
import shlex
import os
import click
import sys

from .metadata_utils import lab_version
from .docker import (
    DockerDaemonInfo,
    RootlessDockerContainer,
    DockerRunBuilder,
    ROOTLESS_DOCKER_NAME
)


def loglevel() -> str:
    return os.environ.get('LOGLEVEL', 'INFO').upper()


logging.basicConfig(level=loglevel())


DISPLAY = os.environ.get('DISPLAY', '')
WORKSPACE = os.environ['LAB_WORKSPACE']
WORKSPACE_DATA = os.path.join(WORKSPACE, 'data')
NETWORK_NAME = 'lab'


@click.command()
def start_cli():
    docker_daemon_info = DockerDaemonInfo.build()
    if not docker_daemon_info.network_exists(NETWORK_NAME):
        docker_daemon_info.create_network(NETWORK_NAME)

    rootless = RootlessDockerContainer(ROOTLESS_DOCKER_NAME, docker_daemon_info)
    if not docker_daemon_info.is_rootless():
        docker_daemon_info = rootless.start_rootless_container(WORKSPACE, NETWORK_NAME)

    cli_cmd = DockerRunBuilder(f'enclarify/lab-partner-cli:{lab_version()}')
    cli_cmd.options() \
        .with_tty() \
        .with_env('ENVIRONMENT', 'LOCAL') \
        .with_env('HOST_DOCKER_SOCKET', docker_daemon_info.docker_internal_socket()) \
        .with_env('LAB_WORKSPACE', os.environ.get('LAB_WORKSPACE')) \
        .with_env('LAB_WORKSPACE', WORKSPACE) \
        .with_env('LAB_WORKSPACE_DATA', WORKSPACE_DATA) \
        .with_env('LAB_NETWORK_NAME', NETWORK_NAME) \
        .with_env('LAB_VERSION', lab_version()) \
        .with_home_dir_bind_mount('.gitconfig', '/opt/lab/home/.gitconfig') \
        .with_home_dir_bind_mount('.vim', '/opt/lab/home/.vim') \
        .with_home_dir_bind_mount('.vimrc', '/opt/lab/home/.vimrc') \
        .with_home_dir_bind_mount('.actrc', '/opt/lab/home/.actrc') \
        .with_home_dir_bind_mount('.aws', '/opt/lab/home/.aws') \
        .with_home_dir_bind_mount('.ssh', '/opt/lab/home/.ssh') \
        .with_bind_mount(WORKSPACE, WORKSPACE) \
        .with_bind_mount('/opt/lab/cicd/artifacts', '/opt/lab/cicd/artifacts') \
        .with_bind_mount(docker_daemon_info.docker_socket(), '/var/run/docker.sock') \
        .with_workdir(WORKSPACE)

    cmd = shlex.split(cli_cmd.build())
    os.execvpe(cmd[0], cmd, {'DOCKER_HOST': f'unix://{docker_daemon_info.docker_socket()}'})


if __name__ == '__main__':
    start_cli()
